var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw new Error('Dynamic require of "' + x + '" is not supported');
});

// src/load/lexer.ts
import { Lexer as Lexer8 } from "chevrotain";

// src/common/environment.ts
var Environment = class {
  constructor() {
    this.debug = false;
  }
  enableDebug() {
    this.debug = true;
  }
  get isDebug() {
    return this.debug;
  }
};
var envs = new Environment();

// src/load/tokens/BasicString.ts
import { createToken as createToken4 } from "chevrotain";

// src/load/tokens/patterns.ts
var XRegExp = __require("xregexp");
var whiteSpaceChar = /[ \t]/;
var nonAscii = /[\x80-\uD7FF]|[\uE000-\uFFFF]/;
var newline = /\r\n|\n/;
var quotationMark = /"/;
var escape = /\\/;
var digit = /[0-9]/;
var hexDigit = XRegExp.build("{{digit}}|[A-Fa-f]", { digit });
var escapeSeqChar = XRegExp.build(
  '["\\\\bfnrt]|u{{hexDigit}}{4}|U{{hexDigit}}{8}',
  { hexDigit }
);
var escaped = XRegExp.build("{{escape}}{{escapeSeqChar}}", {
  escape,
  escapeSeqChar
});
var apostrophe = /'/;
var underscore = /_/;
var minus = /-/;
var plus = /\+/;
var digit1_9 = /[1-9]/;
var unsignedDecimalInteger = XRegExp.build(
  "{{digit1_9}}({{digit}}|{{underscore}}{{digit}})+|{{digit}}",
  {
    digit1_9,
    digit,
    underscore
  }
);
var decimalInteger = XRegExp.build(
  "({{minus}}|{{plus}})?{{unsignedDecimalInteger}}",
  { minus, plus, unsignedDecimalInteger }
);

// src/load/tokens/tokenInterpreters.ts
var tokenInterpreters = /* @__PURE__ */ new Map();
var defaultFallbackInterpreter = (raw, token, category) => {
  const interpreter2 = tokenInterpreters[token.tokenType.name];
  if (interpreter2) {
    return interpreter2(raw, token, category);
  }
  return null;
};
var registerTokenInterpreter = (token, interpreter2) => {
  tokenInterpreters[token.name] = interpreter2;
};

// src/load/exception.ts
var SyntaxParseError = class extends Error {
  constructor(message) {
    super(message);
    this.name = "SyntaxParseError";
  }
};
var LexerError = class extends SyntaxParseError {
  constructor(errors) {
    super("Syntax error\n" + errors.map((error) => error.message).join("\n"));
    this.errors = errors;
  }
};
var ParserError = class extends SyntaxParseError {
  constructor(errors) {
    super("Syntax error\n" + errors.map((error) => error.message).join("\n"));
    this.errors = errors;
  }
};
var InterpreterError = class extends SyntaxParseError {
  constructor(message) {
    super(message);
  }
};

// src/load/tokens/utils.ts
var unescapeString = (string) => {
  let result = "";
  for (let i = 0; i < string.length; i++) {
    const char = string[i];
    if (char === "\\") {
      i++;
      switch (string[i]) {
        case "b":
          result += "\b";
          break;
        case "t":
          result += "	";
          break;
        case "n":
          result += "\n";
          break;
        case "f":
          result += "\f";
          break;
        case "r":
          result += "\r";
          break;
        case '"':
          result += '"';
          break;
        case "\\":
          result += "\\";
          break;
        case "u":
          result += String.fromCharCode(
            parseInt(string.substring(i + 1, i + 5), 16)
          );
          i += 4;
          break;
        case "U":
          result += String.fromCodePoint(
            parseInt(string.substring(i + 1, i + 9), 16)
          );
          i += 8;
          break;
        default:
          throw new InterpreterError(`Invalid escape sequence: \\${string[i]}`);
      }
    } else {
      result += char;
    }
  }
  return result;
};
var getMultiLineContent = (string) => {
  const content = string.substring(3, string.length - 3);
  return content.replace(/^(\r\n|\n)/, "");
};

// src/load/tokens/QuotedKey.ts
import { createToken as createToken2, Lexer as Lexer2 } from "chevrotain";

// src/load/tokens/SimpleKey.ts
import { createToken, Lexer } from "chevrotain";
var SimpleKey = createToken({ name: "SimpleKey", pattern: Lexer.NA });
registerTokenInterpreter(SimpleKey, defaultFallbackInterpreter);

// src/load/tokens/QuotedKey.ts
var QuotedKey = createToken2({
  name: "QuotedKey",
  pattern: Lexer2.NA,
  categories: [SimpleKey]
});
registerTokenInterpreter(QuotedKey, defaultFallbackInterpreter);

// src/load/tokens/TomlString.ts
import { createToken as createToken3, Lexer as Lexer3 } from "chevrotain";
var TomlString = createToken3({ name: "String", pattern: Lexer3.NA });
registerTokenInterpreter(TomlString, defaultFallbackInterpreter);

// src/load/tokens/BasicString.ts
var XRegExp2 = __require("xregexp");
var basicUnescaped = XRegExp2.build(
  "{{whiteSpaceChar}}|!|[#-\\x5B]|[\\x5D-~]|{{nonAscii}}",
  {
    whiteSpaceChar,
    nonAscii
  }
);
var basicChar = XRegExp2.build("{{basicUnescaped}}|{{escaped}}", {
  basicUnescaped,
  escaped
});
var BasicString = createToken4({
  name: "BasicString",
  pattern: XRegExp2.build("{{quotationMark}}{{basicChar}}*{{quotationMark}}", {
    quotationMark,
    basicChar
  }),
  label: '"BasicString"',
  categories: [QuotedKey, TomlString]
});
registerTokenInterpreter(BasicString, (raw) => {
  const value = raw.slice(1, -1);
  return unescapeString(value);
});

// src/load/tokens/Comment.ts
import { createToken as createToken5 } from "chevrotain";
var XRegExp3 = __require("xregexp");
var commentStartChar = /#/;
var nonEol = XRegExp3.build("	|[ -\x7F]|{{nonAscii}}", { nonAscii });
var comment = XRegExp3.build("{{commentStartChar}}{{nonEol}}*", {
  commentStartChar,
  nonEol
});
var Comment = createToken5({
  name: "Comment",
  pattern: comment,
  group: "comment"
});

// src/load/tokens/DotSeparator.ts
import { createToken as createToken6 } from "chevrotain";
var DotSeparator = createToken6({
  name: "DotSeparator",
  pattern: /\./,
  label: "."
});

// src/load/tokens/KeyValueSeparator.ts
import { createToken as createToken7 } from "chevrotain";
var KeyValueSeparator = createToken7({
  name: "KeyValueSeparator",
  pattern: /=/,
  label: "=",
  push_mode: "Value" /* Value */
});

// src/load/tokens/LiteralString.ts
import { createToken as createToken8 } from "chevrotain";
var XRegExp4 = __require("xregexp");
var literalChar = XRegExp4.build("	|[ -&]|[\\x28-~]|{{nonAscii}}", {
  nonAscii
});
var LiteralString = createToken8({
  name: "LiteralString",
  pattern: XRegExp4.build("{{apostrophe}}{{literalChar}}*{{apostrophe}}", {
    apostrophe,
    literalChar
  }),
  label: "'LiteralString'",
  categories: [QuotedKey, TomlString]
});
registerTokenInterpreter(LiteralString, (raw) => raw.slice(1, -1));

// src/load/tokens/MultiLineBasicString.ts
import { createToken as createToken9 } from "chevrotain";
var XRegExp5 = __require("xregexp");
var multiLineBasicStringDelimiter = XRegExp5.build("{{quotationMark}}{3}", {
  quotationMark
});
var multiLineBasicUnescaped = XRegExp5.build(
  "{{whiteSpaceChar}}|!|[#-\\x5B]|[\\x5D-~]|{{nonAscii}}",
  {
    whiteSpaceChar,
    nonAscii
  }
);
var multiLineBasicChar = XRegExp5.build(
  "{{multiLineBasicUnescaped}}|{{escaped}}",
  { multiLineBasicUnescaped, escaped }
);
var whiteSpace = XRegExp5.build("{{whiteSpaceChar}}*", { whiteSpaceChar });
var multiLineBasicEscapedNewline = XRegExp5.build(
  "{{escape}}{{whiteSpace}}{{newline}}({{whiteSpaceChar}}|{{newline}})*",
  {
    escape,
    whiteSpace,
    newline,
    whiteSpaceChar
  }
);
var multiLineBasicContent = XRegExp5.build(
  "{{multiLineBasicChar}}|{{newline}}|{{multiLineBasicEscapedNewline}}",
  {
    multiLineBasicChar,
    newline,
    multiLineBasicEscapedNewline
  }
);
var multiLineBasicQuotes = XRegExp5.build("{{quotationMark}}{1,2}", {
  quotationMark
});
var multiLineBasicBody = XRegExp5.build(
  "{{multiLineBasicContent}}*({{multiLineBasicQuotes}}{{multiLineBasicContent}}+)*{{multiLineBasicQuotes}}?",
  {
    multiLineBasicContent,
    multiLineBasicQuotes
  }
);
var MultiLineBasicString = createToken9({
  name: "MultiLineBasicString",
  pattern: XRegExp5.build(
    "{{multiLineBasicStringDelimiter}}{{newline}}?{{multiLineBasicBody}}{{multiLineBasicStringDelimiter}}",
    {
      multiLineBasicStringDelimiter,
      newline,
      multiLineBasicBody
    }
  ),
  label: '"""MultiLineBasicString"""',
  categories: [TomlString],
  line_breaks: true
});
var skipWhitespaceIfFindBackslash = (string) => string.replace(/\\[ \t]*(\r\n|\n)+[ \t]*/g, "");
registerTokenInterpreter(MultiLineBasicString, (raw) => {
  const content = getMultiLineContent(raw);
  const result = skipWhitespaceIfFindBackslash(content);
  return unescapeString(result);
});

// src/load/tokens/MultiLineLiteralString.ts
import { createToken as createToken10 } from "chevrotain";
var XRegExp6 = __require("xregexp");
var multiLineLiteralStringDelimiter = XRegExp6.build("{{apostrophe}}{3}", {
  apostrophe
});
var multiLineLiteralChar = XRegExp6.build(
  "	|[ -&]|[\\x28-~]|{{nonAscii}}",
  { nonAscii }
);
var multiLineLiteralContent = XRegExp6.build(
  "{{multiLineLiteralChar}}|{{newline}}",
  { multiLineLiteralChar, newline }
);
var multiLineLiteralQuotes = XRegExp6.build("{{apostrophe}}{1,2}", {
  apostrophe
});
var multiLineLiteralBody = XRegExp6.build(
  "{{multiLineLiteralContent}}*({{multiLineLiteralQuotes}}{{multiLineLiteralContent}}+)*{{multiLineLiteralQuotes}}?",
  {
    multiLineLiteralContent,
    multiLineLiteralQuotes
  }
);
var MultiLineLiteralString = createToken10({
  name: "MultiLineLiteralString",
  pattern: XRegExp6.build(
    "{{multiLineLiteralStringDelimiter}}{{newline}}?{{multiLineLiteralBody}}{{multiLineLiteralStringDelimiter}}",
    {
      multiLineLiteralStringDelimiter,
      newline,
      multiLineLiteralBody
    }
  ),
  label: "'''MultiLineLiteralString'''",
  categories: [TomlString],
  line_breaks: true
});
registerTokenInterpreter(MultiLineLiteralString, getMultiLineContent);

// src/load/tokens/Newline.ts
import { createToken as createToken11 } from "chevrotain";
var Newline = createToken11({
  name: "Newline",
  pattern: newline,
  pop_mode: true
});

// src/load/tokens/Boolean.ts
import { createToken as createToken12, Lexer as Lexer4 } from "chevrotain";
var Boolean = createToken12({
  name: "Boolean",
  pattern: Lexer4.NA
});
registerTokenInterpreter(Boolean, defaultFallbackInterpreter);

// src/load/tokens/UnquotedKey.ts
import { createToken as createToken13 } from "chevrotain";
var XRegExp7 = __require("xregexp");
var alpha = /[a-zA-Z]/;
var unquotedKey = XRegExp7.build("({{alpha}}|{{digit}}|-|_)+", {
  alpha,
  digit
});
var UnquotedKey = createToken13({
  name: "UnquotedKey",
  pattern: unquotedKey,
  categories: [SimpleKey]
});
registerTokenInterpreter(UnquotedKey, (raw) => raw);

// src/load/tokens/DecimalInteger.ts
import { createToken as createToken15 } from "chevrotain";

// src/load/tokens/Integer.ts
import { createToken as createToken14, Lexer as Lexer5 } from "chevrotain";
var Integer = createToken14({ name: "Integer", pattern: Lexer5.NA });
registerTokenInterpreter(Integer, defaultFallbackInterpreter);

// src/load/tokens/DecimalInteger.ts
var DecimalInteger = createToken15({
  name: "DecimalInteger",
  pattern: decimalInteger,
  categories: [Integer]
});
registerTokenInterpreter(DecimalInteger, (raw) => {
  const intString = raw.replace(/_/g, "");
  const int = parseInt(intString);
  if (Number.isSafeInteger(int)) {
    return int || 0;
  }
  return BigInt(intString);
});

// src/load/tokens/NonDecimalInteger.ts
import { createToken as createToken16 } from "chevrotain";
var XRegExp8 = __require("xregexp");
var hexPrefix = /0x/;
var octPrefix = /0o/;
var binPrefix = /0b/;
var digit0_7 = /[0-7]/;
var digit0_1 = /[01]/;
var hexInteger = XRegExp8.build(
  "{{hexPrefix}}{{hexDigit}}({{hexDigit}}|{{underscore}}{{hexDigit}})*",
  {
    hexPrefix,
    hexDigit,
    underscore
  }
);
var octalInteger = XRegExp8.build(
  "{{octPrefix}}{{digit0_7}}({{digit0_7}}|{{underscore}}{{digit0_7}})*",
  {
    octPrefix,
    digit0_7,
    underscore
  }
);
var binaryInteger = XRegExp8.build(
  "{{binPrefix}}{{digit0_1}}({{digit0_1}}|{{underscore}}{{digit0_1}})*",
  {
    binPrefix,
    digit0_1,
    underscore
  }
);
var nonDecimalInteger = XRegExp8.build(
  "{{hexInteger}}|{{octalInteger}}|{{binaryInteger}}",
  {
    hexInteger,
    octalInteger,
    binaryInteger
  }
);
var NonDecimalInteger = createToken16({
  name: "NonDecimalInteger",
  pattern: nonDecimalInteger,
  categories: [Integer]
});
var parseBigInt = (string, radix) => {
  let result = BigInt(0);
  for (let i = 0; i < string.length; i++) {
    const char = string[i];
    const digit2 = parseInt(char, radix);
    result = result * BigInt(radix) + BigInt(digit2);
  }
  return result;
};
var getRadix = (raw) => {
  if (raw.startsWith("0x")) {
    return 16;
  } else if (raw.startsWith("0o")) {
    return 8;
  } else if (raw.startsWith("0b")) {
    return 2;
  }
  return 10;
};
registerTokenInterpreter(NonDecimalInteger, (raw) => {
  const intString = raw.replace(/_/g, "");
  const digits = intString.slice(2);
  const radix = getRadix(raw);
  const int = parseInt(digits, radix);
  if (Number.isSafeInteger(int)) {
    return int;
  }
  return parseBigInt(digits, radix);
});

// src/load/tokens/WhiteSpace.ts
import { createToken as createToken17, Lexer as Lexer6 } from "chevrotain";
var XRegExp9 = __require("xregexp");
var whiteSpace2 = XRegExp9.build("{{whiteSpaceChar}}+", { whiteSpaceChar });
var WhiteSpace = createToken17({
  name: "WhiteSpace",
  pattern: whiteSpace2,
  group: Lexer6.SKIPPED
});

// src/load/tokens/Float.ts
import { createToken as createToken18 } from "chevrotain";
var XRegExp10 = __require("xregexp");
var floatIntPart = decimalInteger;
var zeroPrefixableInt = XRegExp10.build(
  "{{digit}}({{digit}}|{{underscore}}{{digit}})*",
  { digit, underscore }
);
var floatExpPart = XRegExp10.build(
  "({{minus}}|{{plus}})?{{zeroPrefixableInt}}",
  { minus, plus, zeroPrefixableInt }
);
var exp = XRegExp10.build("[Ee]{{floatExpPart}}", { floatExpPart });
var decimalPoint = /\./;
var frac = XRegExp10.build("{{decimalPoint}}{{zeroPrefixableInt}}", {
  decimalPoint,
  zeroPrefixableInt
});
var inf = /inf/;
var nan = /nan/;
var specialFloat = XRegExp10.build("({{minus}}|{{plus}})?({{inf}}|{{nan}})", {
  minus,
  plus,
  inf,
  nan
});
var float = XRegExp10.build(
  "{{floatIntPart}}({{exp}}|{{frac}}{{exp}}?)|{{specialFloat}}",
  {
    floatIntPart,
    exp,
    frac,
    specialFloat
  }
);
var Float = createToken18({
  name: "Float",
  pattern: float
});
var attemptSpecialFloat = (value) => {
  if (value === "inf" || value === "+inf") {
    return Infinity;
  }
  if (value === "-inf") {
    return -Infinity;
  }
  if (value === "nan" || value === "+nan" || value === "-nan") {
    return NaN;
  }
  return null;
};
registerTokenInterpreter(Float, (raw) => {
  const value = attemptSpecialFloat(raw);
  if (value !== null) {
    return value;
  }
  const floatString = raw.replace(/_/g, "");
  return parseFloat(floatString);
});

// src/load/tokens/DateTime.ts
import { createToken as createToken19 } from "chevrotain";
var XRegExp11 = __require("xregexp");
var dateFullYear = XRegExp11.build("{{digit}}{4}", { digit });
var dateMonth = XRegExp11.build("{{digit}}{2}", { digit });
var dateMDay = XRegExp11.build("{{digit}}{2}", { digit });
var timeDelim = /[Tt ]/;
var timeHour = XRegExp11.build("{{digit}}{2}", { digit });
var timeMinute = XRegExp11.build("{{digit}}{2}", { digit });
var timeSecond = XRegExp11.build("{{digit}}{2}", { digit });
var timeSecFrac = XRegExp11.build("\\.{{digit}}+", { digit });
var timeNumOffset = XRegExp11.build("[+-]{{timeHour}}:{{timeMinute}}", {
  timeHour,
  timeMinute
});
var timeOffset = XRegExp11.build("[Zz]|{{timeNumOffset}}", { timeNumOffset });
var partialTime = XRegExp11.build(
  "{{timeHour}}:{{timeMinute}}:{{timeSecond}}{{timeSecFrac}}?",
  {
    timeHour,
    timeMinute,
    timeSecond,
    timeSecFrac
  }
);
var fullDate = XRegExp11.build("{{dateFullYear}}-{{dateMonth}}-{{dateMDay}}", {
  dateFullYear,
  dateMonth,
  dateMDay
});
var fullTime = XRegExp11.build("{{partialTime}}{{timeOffset}}", {
  partialTime,
  timeOffset
});
var offsetDateTime = XRegExp11.build("{{fullDate}}{{timeDelim}}{{fullTime}}", {
  fullDate,
  timeDelim,
  fullTime
});
var localDateTime = XRegExp11.build(
  "{{fullDate}}{{timeDelim}}{{partialTime}}",
  { fullDate, timeDelim, partialTime }
);
var localDate = fullDate;
var localTime = partialTime;
var dateTime = XRegExp11.build(
  "{{offsetDateTime}}|{{localDateTime}}|{{localDate}}|{{localTime}}",
  {
    offsetDateTime,
    localDateTime,
    localDate,
    localTime
  }
);
var DateTime = createToken19({
  name: "DateTime",
  pattern: dateTime
});
registerTokenInterpreter(DateTime, (raw) => {
  var _a;
  const onlyTime = ((_a = raw.match(localTime)) == null ? void 0 : _a.index) === 0;
  if (onlyTime) {
    return raw;
  }
  return new Date(raw);
});

// src/load/tokens/ArrayOpen.ts
import { createToken as createToken20 } from "chevrotain";
var arrayOpen = /\[/;
var ArrayOpen = createToken20({
  name: "ArrayOpen",
  pattern: arrayOpen,
  label: "[",
  push_mode: "Array" /* Array */
});

// src/load/tokens/ArraySep.ts
import { createToken as createToken21 } from "chevrotain";
var comma = /,/;
var ArraySep = createToken21({
  name: "Comma",
  pattern: comma,
  label: ","
});

// src/load/tokens/InlineTableOpen.ts
import { createToken as createToken22 } from "chevrotain";
var inlineTableOpen = /{/;
var InlineTableOpen = createToken22({
  name: "InlineTableOpen",
  pattern: inlineTableOpen,
  label: "{",
  push_mode: "InlineTable" /* InlineTable */
});

// src/load/tokens/InlineTableClose.ts
import { createToken as createToken23 } from "chevrotain";
var inlineTableClose = /}/;
var InlineTableClose = createToken23({
  name: "InlineTableClose",
  pattern: inlineTableClose,
  label: "}",
  pop_mode: true
});

// src/load/tokens/True.ts
import { createToken as createToken24 } from "chevrotain";
var truePattern = /true/;
var True = createToken24({
  name: "True",
  pattern: truePattern,
  label: "true",
  categories: [Boolean]
});
registerTokenInterpreter(True, () => true);

// src/load/tokens/False.ts
import { createToken as createToken25 } from "chevrotain";
var falsePattern = /false/;
var False = createToken25({
  name: "False",
  pattern: falsePattern,
  label: "false",
  categories: [Boolean]
});
registerTokenInterpreter(False, () => false);

// src/load/tokens/ArrayClose.ts
import { createToken as createToken26 } from "chevrotain";
var arrayClose = /]/;
var ArrayClose = createToken26({
  name: "ArrayClose",
  pattern: arrayClose,
  label: "]",
  pop_mode: true
});

// src/load/tokens/IgnoredNewline.ts
import { createToken as createToken27, Lexer as Lexer7 } from "chevrotain";
var IgnoredNewline = createToken27({
  name: "IgnoredNewline",
  pattern: newline,
  group: Lexer7.SKIPPED
});

// src/load/tokens/InlineTableSep.ts
import { createToken as createToken28 } from "chevrotain";
var inlineTableSep = /,/;
var InlineTableSep = createToken28({
  name: "InlineTableSep",
  pattern: inlineTableSep,
  label: ",",
  pop_mode: true,
  push_mode: "InlineTable" /* InlineTable */
});

// src/load/tokens/InlineTableKeyValSep.ts
import { createToken as createToken29 } from "chevrotain";
var InlineTableKeyValSep = createToken29({
  name: "InlineTableKeyValSep",
  pattern: /=/,
  label: "=",
  push_mode: "Value" /* Value */,
  pop_mode: true,
  categories: [KeyValueSeparator]
});

// src/load/tokens/StdTableClose.ts
import { createToken as createToken30 } from "chevrotain";
var StdTableClose = createToken30({
  name: "StdTableClose",
  pattern: /]/,
  label: "]",
  push_mode: "Value" /* Value */
});

// src/load/tokens/StdTableOpen.ts
import { createToken as createToken31 } from "chevrotain";
var StdTableOpen = createToken31({
  name: "StdTableOpen",
  pattern: /\[/,
  label: "["
});

// src/load/tokens/ArrayTableOpen.ts
import { createToken as createToken32 } from "chevrotain";
var ArrayTableOpen = createToken32({
  name: "ArrayTableOpen",
  pattern: /\[\[/,
  label: "[["
});

// src/load/tokens/ArrayTableClose.ts
import { createToken as createToken33 } from "chevrotain";
var ArrayTableClose = createToken33({
  name: "ArrayTableClose",
  pattern: /]]/,
  label: "]]",
  push_mode: "Value" /* Value */
});

// src/load/tokens/allTokens.ts
var keyTokens = [
  WhiteSpace,
  BasicString,
  LiteralString,
  UnquotedKey,
  DotSeparator,
  SimpleKey
];
var valueTokens = [
  WhiteSpace,
  MultiLineBasicString,
  MultiLineLiteralString,
  BasicString,
  LiteralString,
  True,
  False,
  DateTime,
  Float,
  NonDecimalInteger,
  DecimalInteger,
  ArrayOpen,
  InlineTableOpen,
  Comment
];
var allTokens = {
  modes: {
    ["Key" /* Key */]: [
      Comment,
      IgnoredNewline,
      KeyValueSeparator,
      ArrayTableOpen,
      ArrayTableClose,
      StdTableOpen,
      StdTableClose,
      ...keyTokens
    ],
    ["Value" /* Value */]: [...valueTokens, Newline, InlineTableSep, InlineTableClose],
    ["Array" /* Array */]: [...valueTokens, IgnoredNewline, ArraySep, ArrayClose],
    ["InlineTable" /* InlineTable */]: [...keyTokens, InlineTableKeyValSep, InlineTableClose]
  },
  defaultMode: "Key" /* Key */
};

// src/load/lexer.ts
var lexer = new Lexer8(allTokens, {
  ensureOptimizations: true,
  skipValidations: !envs.isDebug
});

// src/load/parser.ts
import { CstParser } from "chevrotain";
var Parser = class extends CstParser {
  constructor() {
    super(allTokens);
    this.dottedKey = this.RULE("dottedKey", () => {
      this.CONSUME(SimpleKey);
      this.AT_LEAST_ONE(() => {
        this.CONSUME(DotSeparator);
        this.CONSUME1(SimpleKey);
      });
    });
    this.key = this.RULE("key", () => {
      this.OR([
        { ALT: () => this.SUBRULE(this.dottedKey) },
        { ALT: () => this.CONSUME(SimpleKey) }
      ]);
    });
    this.inlineTableKeyValues = this.RULE("inlineTableKeyValues", () => {
      this.MANY_SEP({
        SEP: InlineTableSep,
        DEF: () => this.SUBRULE(this.keyValue)
      });
    });
    this.inlineTable = this.RULE("inlineTable", () => {
      this.CONSUME(InlineTableOpen);
      this.OPTION(() => this.SUBRULE(this.inlineTableKeyValues));
      this.CONSUME(InlineTableClose);
    });
    this.array = this.RULE("array", () => {
      this.CONSUME(ArrayOpen);
      this.OPTION(() => this.SUBRULE(this.arrayValues));
      this.CONSUME(ArrayClose);
    });
    this.value = this.RULE("value", () => {
      this.OR([
        { ALT: () => this.CONSUME(TomlString) },
        { ALT: () => this.CONSUME(Boolean) },
        { ALT: () => this.SUBRULE(this.array) },
        { ALT: () => this.SUBRULE(this.inlineTable) },
        { ALT: () => this.CONSUME(DateTime) },
        { ALT: () => this.CONSUME(Float) },
        { ALT: () => this.CONSUME(Integer) }
      ]);
    });
    this.keyValue = this.RULE("keyValue", () => {
      this.SUBRULE(this.key);
      this.CONSUME(KeyValueSeparator);
      this.SUBRULE(this.value);
    });
    this.arrayValues = this.RULE("arrayValues", () => {
      this.SUBRULE(this.value);
      let havingMore = true;
      this.MANY({
        GATE: () => havingMore,
        DEF: () => {
          this.CONSUME(ArraySep);
          const found = this.OPTION(() => this.SUBRULE1(this.value));
          if (!found) {
            havingMore = false;
          }
        }
      });
    });
    this.stdTable = this.RULE("stdTable", () => {
      this.CONSUME(StdTableOpen);
      this.SUBRULE(this.key);
      this.CONSUME(StdTableClose);
    });
    this.arrayTable = this.RULE("arrayTable", () => {
      this.CONSUME(ArrayTableOpen);
      this.SUBRULE(this.key);
      this.CONSUME(ArrayTableClose);
    });
    this.table = this.RULE("table", () => {
      this.OR([
        { ALT: () => this.SUBRULE(this.stdTable) },
        { ALT: () => this.SUBRULE(this.arrayTable) }
      ]);
    });
    this.expression = this.RULE("expression", () => {
      this.OR([
        { ALT: () => this.SUBRULE(this.keyValue) },
        { ALT: () => this.SUBRULE(this.table) }
      ]);
    });
    this.toml = this.RULE("toml", () => {
      this.OPTION(() => this.SUBRULE(this.expression));
      this.MANY(() => {
        this.CONSUME(Newline);
        this.SUBRULE1(this.expression);
      });
      this.OPTION1(() => this.CONSUME1(Newline));
    });
    this.performSelfAnalysis();
  }
};
var parser = new Parser();

// src/load/interpreter.ts
var isPlainObject = (obj) => obj && obj.constructor === Object;
var tryCreateKey = (operation, message) => {
  try {
    return operation();
  } catch (error) {
    if (error instanceof DuplicateKeyError) {
      throw new InterpreterError(message);
    }
  }
};
var DuplicateKeyError = class extends Error {
};
var BaseCstVisitor = parser.getBaseCstVisitorConstructor();
var tableDeclared = Symbol("tableDeclared");
var notEditable = Symbol("notEditable");
var Interpreter = class extends BaseCstVisitor {
  constructor() {
    super();
    this.validateVisitor();
  }
  toml(ctx) {
    var _a;
    const root = {};
    let current = root;
    (_a = ctx.expression) == null ? void 0 : _a.forEach(
      (expression) => current = this.visit(expression, { current, root })
    );
    this.cleanInternalProperties(root);
    return root;
  }
  expression(ctx, { current, root }) {
    if (ctx.keyValue) {
      this.visit(ctx.keyValue, current);
      return current;
    } else if (ctx.table) {
      return this.visit(ctx.table, root);
    }
  }
  keyValue(ctx, object) {
    const keys = this.visit(ctx.key);
    const value = this.visit(ctx.value);
    tryCreateKey(
      () => this.assignValue(keys, value, object),
      `Cannot assign value to '${keys.join(".")}'`
    );
  }
  key(ctx) {
    if (ctx.dottedKey) {
      return this.visit(ctx.dottedKey);
    } else {
      return [this.interpret(ctx, SimpleKey)];
    }
  }
  dottedKey(ctx) {
    return this.interpret(ctx, SimpleKey);
  }
  inlineTableKeyValues(ctx, object) {
    if (ctx.keyValue) {
      ctx.keyValue.forEach((keyValue) => this.visit(keyValue, object));
    }
  }
  inlineTable(ctx) {
    const result = { [notEditable]: true };
    if (ctx.inlineTableKeyValues) {
      this.visit(ctx.inlineTableKeyValues, result);
    }
    return result;
  }
  value(ctx) {
    if (ctx.array) {
      return this.visit(ctx.array);
    } else if (ctx.inlineTable) {
      return this.visit(ctx.inlineTable);
    }
    return this.interpret(ctx, TomlString, Float, Boolean, DateTime, Integer);
  }
  arrayValues(ctx, array) {
    ctx.value.forEach((value) => array.push(this.visit(value)));
    return array;
  }
  array(ctx) {
    const result = [];
    result[notEditable] = true;
    if (ctx.arrayValues) {
      return this.visit(ctx.arrayValues, result);
    }
    return result;
  }
  table(ctx, root) {
    if (ctx.stdTable) {
      return this.visit(ctx.stdTable, root);
    } else if (ctx.arrayTable) {
      return this.visit(ctx.arrayTable, root);
    }
  }
  stdTable(ctx, root) {
    const keys = this.visit(ctx.key);
    return tryCreateKey(
      () => this.createTable(keys, root),
      `Cannot create table '${keys.join(".")}'`
    );
  }
  arrayTable(ctx, root) {
    const keys = this.visit(ctx.key);
    return tryCreateKey(() => {
      const array = this.getOrCreateArray(keys, root);
      if (array[notEditable]) {
        throw new DuplicateKeyError();
      }
      const object = {};
      array.push(object);
      return object;
    }, `Cannot create array table '${keys.join(".")}'`);
  }
  cleanInternalProperties(object) {
    for (const symbol of Object.getOwnPropertySymbols(object)) {
      delete object[symbol];
    }
    for (const key in object) {
      if (typeof object[key] === "object") {
        this.cleanInternalProperties(object[key]);
      }
      if (Array.isArray(object[key])) {
        object[key].forEach((item) => this.cleanInternalProperties(item));
      }
    }
  }
  interpret(ctx, ...candidates) {
    for (const type of candidates) {
      if (ctx[type.name]) {
        const result = ctx[type.name].map(
          (token) => tokenInterpreters[type.name](token.image, token, type.name)
        );
        return result.length === 1 ? result[0] : result;
      }
    }
    return null;
  }
  assignPrimitiveValue(key, value, object) {
    if (object[key]) {
      throw new DuplicateKeyError();
    }
    if (isPlainObject(value)) {
      value[tableDeclared] = true;
    }
    object[key] = value;
    return object;
  }
  tryCreatingObject(key, object, declareTable, ignoreDeclared) {
    if (object[key]) {
      if (!isPlainObject(object[key]) || !ignoreDeclared && object[key][tableDeclared] || object[key][notEditable]) {
        throw new DuplicateKeyError();
      }
    } else {
      object[key] = {};
      if (declareTable) {
        object[key][tableDeclared] = true;
      }
    }
    return object[key];
  }
  assignValue(keys, value, object) {
    const [first, ...rest] = keys;
    if (rest.length > 0) {
      this.tryCreatingObject(first, object, true, true);
      return this.assignValue(rest, value, object[first]);
    }
    return this.assignPrimitiveValue(first, value, object);
  }
  createTable(keys, object) {
    const [first, ...rest] = keys;
    if (rest.length > 0) {
      if (Array.isArray(object[first])) {
        const toAdd = object[first][object[first].length - 1];
        return this.createTable(rest, toAdd);
      }
      this.tryCreatingObject(first, object, false, true);
      return this.createTable(rest, object[first]);
    }
    return this.tryCreatingObject(first, object, true, false);
  }
  getOrCreateArray(keys, object) {
    const [first, ...rest] = keys;
    if (rest.length > 0) {
      if (Array.isArray(object[first])) {
        const toAdd = object[first][object[first].length - 1];
        return this.getOrCreateArray(rest, toAdd);
      }
      this.tryCreatingObject(first, object, false, true);
      return this.getOrCreateArray(rest, object[first]);
    }
    if (object[first] && !Array.isArray(object[first])) {
      throw new DuplicateKeyError();
    }
    object[first] = object[first] || [];
    return object[first];
  }
};
var interpreter = new Interpreter();

// src/load/load.ts
var load = (toml) => {
  const lexingResult = lexer.tokenize(toml);
  if (lexingResult.errors.length > 0) {
    throw new LexerError(lexingResult.errors);
  }
  parser.input = lexingResult.tokens;
  const cst = parser.toml();
  if (parser.errors.length > 0) {
    throw new ParserError(parser.errors);
  }
  return interpreter.visit(cst);
};
export {
  SyntaxParseError,
  load
};
